package TableLoader;
use strict;
use DBI;
use Table::Denormalized;

=head1 NAME

TableLoader - create and load a table from a CSV file for tests

=head1 SYNOPSIS

  use lib 'inc';
  use TableLoader;
  my $dbh = create_and_load("my_table",\*DATA);
  
  create_and_load("another_table", $fh, $dbh);
  
This module will create and load a temporary
SQLite database. The file of the database
will be C<$0.sqlite>, and that file will be
deleted at program exit time in an C<END> block.
  
=head2 C<< create_and_load TABLE, FH, PRIMARY >>

Creates a table and loads it from the given
file handle. The file must have a specific structure:

=over 4

=item * All items in the line must be comma separated

=item * The first line must contain the column headers 
and column types:

  id:INTEGER PRIMARY KEY,first_name:VARCHAR(200),last_name:VARCHAR(200)

No fancy quoting constructs are allowed. This module is for easy
testing - if you want proper CSV loading, look at L<Text::CSV_XS>
or any of the other CSV modules.

=item * All following lines are the comma separated
values for the columns.

=cut

my @cleanup;

sub create_and_load {
  my ($name,$fh,$primary) = @_;
  $primary ||= 'uri';

  my $filename = "$0.sqlite";
  if (-f $filename) {
    unlink $filename or
      warn "Couldn't remove '$filename': $!";
  };
    
  unshift @cleanup, sub {
    unlink $filename
      or warn "Couldn't remove '$filename' : $!";
  };
  
  # set up test database
  my $headers = <$fh>;
  chomp $headers;
  my %columns = split /[:,]/, $headers;
  my @columns = ($headers =~ /(?:^|,)([^,:]+).*?/g);

  my $table = Table::Denormalized->new(
    dsn => "dbi:SQLite:$filename",
    columns => [ %columns ],
    table => $name,
  );
  $table->create;
  $table->alter_table;

  while (<$fh>) {
    chomp;
    my %v;
    @v{@columns} = split /,/;
    for (keys %v) {
        if ($v{$_} eq 'NULL') {
            undef $v{$_};
        }
    }
    $table->update_or_insert( \%v, $primary );
  };
  
  close $fh;
  
  $table
};

END {
  for (@cleanup) { $_->() };
};

sub import {
  my $caller = caller;
  no strict 'refs';
  *{"$caller\::create_and_load"} = \&create_and_load;
};

1;