package FFRIndexed;
use Table::Denormalized;
use File::Find ();
use DBI;
use Data::Dumper;
use Memoize;
use Scalar::Util qw(weaken);
use FFRIndexed::File;
use Carp qw(croak);
use base 'Table::Denormalized::Manager';

use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

FFRIndexed - SQL Index for your files

=head1 SYNOPSIS

In the near future, there should be a File::Find::Rule-like
interface to the SQL database (hence the ugly prefix FFR),
but for now you need to construct queries conforming
to the whims of the SQL::Abstract library.

  use FFRIndexed;
  my $idx = FFRIndexed->new( create_if_needed => 1 );

  $idx->scan('/home/media/mp3/');
  # this will rescan all files and stat them ...

  my $files = $idx->query( { mime_type => 'audio/mpeg', artist => 'Jamiroquai' });
  printf "%s - %s - %s\n", $_->album, $_->title, $_->uri for @$files;

=cut

use Module::Pluggable::Fast
  name   => 'plugins',
  ;

memoize('plugins');

=head1 PLUGINS

The main module itself provides almost no functionality
and delegates all the heavy lifting to plugins.

There are some remaining questions about plugins:

=over 2

=item Can/should we reorder the plugins so the
two well-known plugins "File" and "MimeType" come first?

=back

=cut

=head2 C<< __PACKAGE__->new >>

Creates a new instance of an index. The following parameters
are accepted:

=head3 C<scan>

Forces a rescan of all directories (or files?)

=head3 C<create>

Creates the table. Wipes all data from the table
should it already exist.

=head3 C<create_if_needed>

Creates the table if needed.

=head3 C<table>

The name of the table where the index gets stored in. Default
is C<fileindex>.

=head3 C<dsn>

The DBI DSN to use. Default is C<dbi:SQLite:dbname=$0.sqlite>.

If you want an alternate database to be used,
you can change that here, or with the C<dbh> parameter,
if you already have a database handle.

=head3 C<dbh>

If you want to provide an alternate database
other than SQLite, you can supply the C<$dbh> you get
from a DBI connect call.

=head3 C<row_class>

The class into which returned rows are blessed into.
This allows you to specify a L<Class::DBI> or L<DBIx::Class>
subclass. Default is C<FFRIndexed::File>, which is a
lightweight Class::DBI reimplementation.

=head3 C<verbose>

Outputs actions to the screen.

=cut

sub new {
    my ($class,%args) = @_;

    $args{table} ||= 'fileindex';
    $args{row_class} ||= 'FFRIndexed::File';

    my $scan;
    if (exists $args{scan}) {
        $scan = delete $args{scan};
        if (! ref $scan) {
            $scan = [ $scan ];
        };
    };

    if ($args{verbose}) {
        warn "Found plugin " . ref $_
            for plugins;
    };

    $args{ columnset } ||= [ map { $_->columnset } plugins() ];
    my $self = $class->SUPER::new(\%args);

    my $verbose = $self->verbose;

    if ($scan) {
        if ($verbose) {
            warn "Scanning @$scan";
            $self->{file_callback} ||= sub{ print "Scanning $_[0] ($_[1])\n" };
        };
        $self->scan(@$scan);
    };

    $self
};

sub _iterate_list {
  my $self = shift;
  my $leaf_callback = shift;
  local $self->table->dbh->{AutoCommit} = 0;
  for my $dir (@_) {
    my $uri = $dir;
    $uri = URI->new($dir)->file
      if ($dir =~ m!^file://!);
    $self->_iterate_dir($leaf_callback, $uri);
    $self->table->dbh->commit;
  };
};

sub _iterate_dir {
  my $self = shift;
  my $leaf_callback = shift;
  File::Find::find( { wanted => sub { -f && $leaf_callback->( URI::file->new_abs($File::Find::name) ) }, no_chdir => 1 }, @_ );
};

sub scan_force {
  my ($self) = shift;
  $self->_iterate_list(sub { my $self = shift; $self->scan_force_file(@_) });
};

sub scan_force_file {
  my ($self,$name) = @_;
  $name = $name->file;
  my $knowledge = $self->get_file_info($name);
  # First look if we know that file already, or another of the same name
  # by its MD5. If we know it, and the other file is dead, update the other
  # file.
  if ($knowledge->{uri}) {
    # It's a new file
    $self->table->update_or_insert($knowledge,'uri');
  } else {
    # We should try to delete the (stale, moved/deleted) entry, some day
  };
  $knowledge;
};

sub scan_file {
  my ($self,$name) = @_;
  die "Cannot scan a non-file URI ($name)"
    unless $name =~ /^file/;

  my $file = $self->query({ uri => "$name" });

  # What do we do about non-existent files?
  # Somehow they should be re-merged with their new-found MD5 matches, maybe.
  # This could be done in the MD5 plugin - if there is an MD5 match and {uri}
  # is missing, appropriate all information from the matched file and check
  # for staleness again (that is, set the {uri} of the found row and load
  # all data of that row into %$knowledge, checking against what we have already
  # for identity)

  if (! @$file) {
    $self->{file_callback}->($name,"new")
      if $self->{file_callback};
    return $self->scan_force_file( $name )
  } else {
    die "Multiple files slurped for uri '$name'"
      if @$file > 1;
    $self->{file_callback}->($name,"refresh")
      if $self->{file_callback};
    return $file->[0]->refresh
  };
  return $file->[0]
};

sub scan {
  my ($self) = shift;
  $self->_iterate_list(sub { $self->scan_file(@_) }, @_);
};

sub get_file_info {
  my ($self,$name) = @_;
  my $knowledge = {};
  if (-f $name) {
    my $file = IO::Handle::Lazy->new($name);
    for my $plugin (plugins) {
      $plugin->scan($file,$knowledge);
    };
  };
  $knowledge
};

=head2 C<< stale INFO >>

Returns a list of columns in the database
that are to be considered stale for the hash
reference C<INFO>. The returned elements of the
list are hashes with the keys C<plugin>,
C<column> and, if applicable, C<new_value>
and C<stale_value>.

=cut

sub stale {
  my ($self,$info) = @_;
  my $knowledge = {};
  my @res;

  my $file_moniker = $info->{uri};
  if ($file_moniker =~ /^file:/) {
    $file_moniker = URI->new($file_moniker)->file;
  };

  my $file = IO::Handle::Lazy->new($file_moniker);
  for my $plugin (plugins) {
    my ($is_stale,$stale_column,$new_value) = $plugin->stale($file,$info);
    if ($is_stale) {
      push @res, { plugin => $plugin,
                   column => $stale_column,
                   defined $new_value ? (new_value => $new_value, stale_value => $info->{$stale_column}) : () };
    };
  };
  @res;
};

=head2 C<< find PARAMS >>

This will (one day) become the L<File::Find::Rule>
drop-in replacement. Not implemented yet.

=cut

sub find {
  my ($self,$query) = @_;
  croak "find is not implemented yet.";
  # $self->table->
};

1;

=head1 AUTHOR

Max Maischein, E<lt>corion@cpan.orgE<gt>

=head1 SEE ALSO

L<File::Find::Rule>, L<Class::DBI>, L<Table::Denormalized>, L<SQL::Abstract::Clauses>
