package FFRIndexed::Template::Plugin;
use strict;
use base 'Class::Data::Inheritable';
use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

FFRIndexed::Template::Plugin - plugin base class for results

=head1 SYNOPSIS

  package FFRIndexed::Template::Plugin::MyList;
  use strict;
  use base 'FFRIndexed::Template::Plugin';

  # See below for methods to override

=head1 ABSTRACT

These plugins are for providing additional links
and rendering the resultsets according to these
additional links.

Maybe later on there will be a plugin for
rendering the main body
of the resultset, and its surroundings.

But not yet.

The class derives from L<Class::Accessor>, thus its
constructor needs to be called with a hash refernce.

There are three accessors:

=head2 C<< visual >>

Returns the text to display to the user.

=head2 C<< content_type >>

Returns the content type header to send
for the item.

The default is C<text/html>

=head2 C<< types >>

Returns a hash reference of the mapping between
locations and templates.

The currently well known locations are:

  page         A complete page on its own
  summary      A summary just above the table

The default is C<page => 'query'>, so it likely should
be overridden.

=cut

__PACKAGE__->mk_classdata('visual', 'generic plugin' );
__PACKAGE__->mk_classdata('content_type', 'text/html');
__PACKAGE__->mk_classdata('types', {
  page => 'query',
});

sub new {
  my $class = shift;
  my $self = bless {}, $class;
  $self;
};

=head2 C<< types >>

Holds the mapping of types to templates.
The template will be either included or run
as its own file, depending on the type.

The current well-known types are

  page      A complete page
  summary   Stuff rendered above the table paging the current collection
  item      Stuff rendered for a single item

=head2 C<< applies QUERY, RESULTSET, REFINEMENTS, ROW >>

Should return true if the plugin applies to the
current result set.

In the base class, this always returns true
so any deriving plugin should override this.

The C<ROW> entry is undef for the C<page>
or C<summary> render types.

=cut

sub applies { 1 };

=head2 C<< template CLASS >>

Returns the template file to use/include. The
template will be called with at least the parameter
C<query>.

=cut

sub template {
  my ($self, $page) = @_;
  $page ||= 'page';
  $self->types->{$page}
};

1;
