package FFRIndexed::Template::Plugin::Album;
use strict;
use base 'FFRIndexed::Template::Plugin';
use Memoize;
use vars qw($VERSION);
$VERSION = '0.06';

memoize('find_album_image');

=head1 NAME

FFRIndexed::Template::Plugin::Album - display album image

=cut

__PACKAGE__->visual( 'as album image' );
__PACKAGE__->types({
  summary => 'album',
  item    => 'image_thumbnail',
});

=head1 DESCRIPTION

An album image can be displayed if the current collection
contains only audio files belonging to the same album
(that is, to the same album title) and an image
exists that also has that same title.

=cut

sub applies {
  my ($self, $query, $resultset, $refinements, $row) = @_;
  # warn "Checking if " . __PACKAGE__ . " applies.";

  if ($row) {
    # Should also restrict the size to the thumbnail:
    #  width => { '<' => 100 }
    return $self->album_image($row->_index, $row->album, $row->artist);
  } else {
    return $self->find_album_image($query, $resultset, $refinements);
  };
};

sub find_album_image {
  my ($self, $query, $resultset, $refinements) = @_;

  # If the resultset is empty, we don't apply
  return
    unless $resultset;

  # Now, (manually) scan our current resultset.
  # This possibly should be done via the database...
  my @tracks = grep { ($_->mime_type || "") =~ m!^audio/|/ogg$! } @$resultset;

  # If we don't have any tracks, we don't apply
  return
    unless @tracks;

  my $album = $tracks[0]->album;

  return
    if grep { uc $_->album ne uc $album } @tracks;

  # Let's just pretend that there will only be one (relevant) artist
  my $artist = $tracks[0]->artist;

  return
    if grep { uc $_->artist ne uc $artist } @tracks;

  return $self->album_image($tracks[0]->_index, $album, $artist);
};

=head2 C<< album_image ALBUM >>

Tries to find a good matching album image.

Currently, this ignores the artist completely, which
is horribly wrong. Also, this method should rank
the images in quality to give better results.

=cut

sub album_image {
  my ($self, $table, $album, $artist) = @_;
  my @artist;
  if ($artist) {
    @artist = ( artist => { -like => $artist });
  };

  my $images = eval {
     $table->query_ex( where => [{ album => { -like => $album }, mime_type => 'image/jpeg' },
                    { album => '', title => { -like => "$album%" }, @artist, mime_type => 'image/jpeg' },
                    { album => $album, @artist, mime_type => 'image/jpeg' }
                    ], order => "width desc" );
  };

  # Select the appropriate quality
  # and the "best" match

  $images->[0]
};

1;
