package HTTP::Server::FFRIndexed;
use strict;
use base qw(HTTP::Server::Simple::CGI Class::Accessor);
use HTTP::Server::Simple::Static;
use FFRIndexed;
use FFRIndexed::Template;
use Template;
use Data::Dumper;
use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

HTTP::Server::FFRIndexed - HTTP server for the file index

=head1 SYNOPSIS

  #!/usr/bin/perl -w
  use strict;
  use HTTP::Server::FFRIndexed;

  HTTP::Server::FFRIndexed
    ->new(columns => [qw[mime_type artist album track title ]],
        dsn => "dbi:SQLite:dbname=$0.sqlite",
        scan => ["C:/Audio Captures/",],
        verbose => 1,
    )->run;

=head1 ABSTRACT

This module implements the controller and server for
a L<FFRIndexed> database.
It should maybe be split up into the real
(HTTP) server and the controller if possible.

=head1 CAVEATS

The server does currently not enforce Taint Mode and
doesn't really validate its input parameters, so it
is susceptible both to path attacks and cross site
scripting attacks or HTML injection attacks.

=cut

__PACKAGE__->mk_accessors(qw(index columns template pp));

=head2 C<< __PACKAGE__->new ARGS >>

Creates a new server instance. The following arguments
are recognized:

=head3 C<< port >>

The TCP port to listen on

=head3 C<< template >>

The template directory

=head3 C<< columns >>

The columns to display

=cut

sub new {
  my ($package,%args) = @_;

  my $port = (delete $args{port}) || 80;
  my $self = $package->SUPER::new($port);
  my $template = delete $args{template}
                 || Template->new({
                      INCLUDE_PATH => 'templates/',
                      DEBUG => 1,
                      POST_CHOMP => 1 });

  $self->pp( CGI::PersistentParams->new(
    discard => [qw[action _id super_search Search column template ss _]],
  ));

  my $columns = delete $args{columns};

  $self->index(FFRIndexed->new(%args));

  $self->columns($columns);
  $self->template($template);

  $self;
};

my %row_method = (
  purge => 'delete',
  rescan => 'rescan',
);

my %server_method = (
  import => 'scan',
  # put super search here?
);

sub handle_request {
  my ($self,$cgi) = @_;

  $self->pp->cgi($cgi);

  my $q = $ENV{PATH_INFO} || "";
  if ($q =~ s!/template:(\w+)/!/!) {
    $cgi->param('template' => $1);
  };
  warn "Request for ($q) / " . $cgi->url('-query' => 1, '-absolute' => 1);

  my @search_columns; # will be filled in case of super_search with specified columns

  $q = "/static$q"
    if ($q eq '/favicon.ico');

  # Ugly hack
  my $any;
  $any = uri_decode( $1 )
    if ($q =~ m!/any:([^/]*)/$!);
  warn "Any: $any"
    if $any;

  if ($q =~ m!^/static/!) {
    if ($q =~ m!^/static/([-\w_][-\w_.]+)$! && -f "static/$1") {
      warn "Serving static/$1";
      $self->serve_static($cgi,'.');
      return
    } else {
      warn "Request for illegal filename '$q', ignored.";
    };
  } elsif ($q =~ m!^/media/(\d+)$!) {
    # My own (re)implementation of serve_static()
    # for arbitrary files
    my $id = $1;
    my $files = $self->index->retrieve( $id );
    # warn Dumper $files;
    if (@$files) {
      my $file = $files->[0];
      # warn $files->[0]->filename;
      open my $fh, "<", $file->filename
        or do {
          print "HTTP/1.1 403 ACCESS DENIED\r\n\r\n";
          return
        };
      print "HTTP/1.1 200 OK\r\n";
      print "Content-Size: " . $file->size . "\r\n";
      print "Content-Type: " . $file->mime_type . "\r\n";
      print "Title: " . $file->title . "\r\n"
        if $file->title;
      print "\r\n";
      binmode $fh;
      binmode STDOUT;
      while (read $fh, my $buffer, 512*1024) {
        print $buffer;
      };
    } else {
      print "HTTP/1.1 404 NOT FOUND\r\n\r\n";
    };
    return
  } elsif ($cgi->param("action")) {
    my $action = $cgi->param("action");

    if (exists $row_method{$action}) {
      my $method = $row_method{$action};
      # maybe some sanity/access check here?

      my @rows = $cgi->param("_id"); # make _id variable
      my $rows = $self->index->retrieve( @rows );
      for my $row (@$rows) {
        # warn Dumper $row;
        $row->$method
          if ($row);
      };
    } elsif (exists $server_method{$action}) {
      my $method = $server_method{$action};
      my @args = $cgi->param("args");
      # maybe some sanity/access check here?

      $self->index->$method(@args);
    };
  } elsif ($cgi->param("super_search")) {
    if ($q !~ m!/$!) { $q .= "/" };
    my $ss = $cgi->param("super_search");
    @search_columns = $cgi->param('column');
    #$cgi->delete($_)
    #  for qw(super_search Search column);

    if (! @search_columns) { # plain super_search
      my @args;
      if (defined $cgi->param("_")) {
        push @args, template => 'auto_complete'
      };
      my $query_string = $self->pp->link({ @args });
      $query_string = $query_string ? "?$query_string" : "";
      $q .= "any:" . $ss . "/" . $query_string;
      warn "Redirecting to $q";
      print "HTTP/1.1 302 MOVED\r\n";
      print $cgi->redirect($q);
      return;
    } else {
      # This is likely an autocompletion search.
      # The default action will handle it by using the right
      # template and everything
      $any = $ss;
    };
  };

  my %template_args;
  my $template_file;

  for (qw(refinement)) {
    if (defined $cgi->param($_)) {
      $template_args{$_} = $cgi->param($_);
      # $cgi->delete($_);
    };
  };

  my ($query,$parts) = $self->decode_query($q);
  return unless $query;

  my %explicit = grep { !/^any$/ } map { /^(.*?):/ ? ($1 => 1) : () } @$parts;

  %template_args = (
     %template_args,
     DEBUG => 1,
     query_string => $q,
     query_root => "",
     query_parts => $parts,
     any_query => $any, # contains the last any: query, if any
  );

  my %params;
  for (qw( page items_per_page order )) {
    $params{$_} = $cgi->param($_);
  };
  $template_args{column} = \@search_columns;

  my $template = $cgi->param("template");
  #warn "Proposed template $template";

  $template_args{query} = FFRIndexed::Template->new(
    index => $self->index,
    query => $query,
    %params,
    columns => $self->columns,
    order => $params{order},
    explicit_columns => [ sort keys %explicit ],
    sticky => [ keys %params ], # still doesn't do anything
    template => $template,
  );

  $template_file ||= 'query.template';

  print "HTTP/1.1 200 OK\r\n";
  if ($template && $template =~ /^(\w+)$/) {
    # This logic here to find the template class+filename is weird :)
    (my $classname = $template) =~ tr!_!!d;
    (my $class) = grep { (ref $_) =~ /::\Q$classname\E$/i } $template_args{query}->_plugins;
    if ($class) {
      $template_file = $class->template . ".template";
      undef $template_file
        if (! -f "templates/$template_file");
      warn "Switching template to '$template_file'";
      print $cgi->header( -content_type => $class->content_type );
    };
  } else {
    print "Content-Type: text/html\r\n";
  };
  print "\r\n";

  $self->template->process( $template_file, \%template_args )
    or die $self->template->error;
};

sub uri_decode {
  my (@vals) = @_; # to localize the variables
  my @res = map { s!%([a-f\d]{2})!chr(hex $1)!eig; $_ } @vals;
  wantarray ? @res : $res[0]
};

# Make this one return the explicit columns too!
sub decode_query {
  my ($self,$query) = @_;

  $query =~ s!^/!!;
  return if $query =~ /\bfavicon.ico$/i;

  my @and = map { uri_decode $_ } split qr!/!, $query;
  my @cat = map { /^(.+?):(.*)$/ ? +{ $1 => $2 } : +{ 'artist' => $_ } } @and;
  for (@cat) {
    my ($k,$v) = %$_;
    $v = [ split qr/;/, $v ]
      if ($v =~ /;/);
    $_ = +{ $k => $v }; # convert to map

    if ($k eq 'any') {
      my $arg = $v;

      # Make this a property of FFRIndexed::Template
      my $ss_columns = [qw[title album artist]];

      # ugly hack just to get it to work NOW ;)
      # The SuperSearch should really, really be
      # completely separate. It's not even a proper
      # sensible subclass of SQL::Abstract::Clauses
      $_ = $self->index->table->sql->super_search("%$arg%",$ss_columns);
    };
  };
  my $result = {};
  if (@cat) {
    $result = [ -and => \@cat ];
  };

  warn "Query structure:";
  warn Dumper $result;
  $result,\@and
};

1;
