package SQL::Abstract::SuperSearch;
use strict;
use base 'SQL::Abstract::Clauses';
use Carp qw(croak);
use Data::Dumper;

use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

SQL::Abstract::SuperSearch - search a value in many columns

=head1 SYNOPSIS

  my ($sql,@bind) = $s->select($table,[ 'listened', 'count(*) as col_count' ],
                          super_search => 'Jamiroquai%',
                          super_search_columns => [qw[artist album track]],
                          where => { mime_type => 'audio/mpeg' },
                          group_by => 'listened',
                          order_by => "col_count desc",
                    );

will generate the SQL statement

  SELECT listened, count(*) as col_count
  FROM files
    WHERE (( mime_type = ? )
       AND ( artist like ?
          OR album like  ?
          OR track like  ? ))
    GROUP BY listened
    ORDER BY col_count desc

and C<@bind> will contain the appropriate bind values:

  # @bind = ('audio/mpeg', 'Jamiroquai%','Jamiroquai%','Jamiroquai%')

=head1 CAVEATS

This module adds cross-column searching
capabilities. It doesn't take into accoubnt
the types of columns, so that's the callers'
responsibility.

=head2 C<< super_search VALUE, COLUMNS, WHERE >>

This is the core implementation. See the
synopsis for some example.

=cut

sub super_search {
  my ($self,$value,$columns,$where,$column_op) = @_;

  $columns = [ $columns ]
    unless ref $columns;

  $column_op ||= "like";

  croak "No columns to search"
    unless @$columns;

  my $column_clause = [ -nest => [ map { $_ => { $column_op => $value }} @$columns ]];

  if ($where) {
    return [ -and => [ $column_clause, $where ]];
  } else {
    return $column_clause;
  };
};

=head2 C<< $sql->select TABLE, COLUMNS, %CLAUSES >>

Returns the SQL statement and bind values for the clauses. This
is B<different> from how L<SQL::Abstract> does it. The code
tries to recognize old usage and fall back to SQL::Abstract,
but the fallback doesn't always work.

  my ($sql,@bind) = $s->select('files',[ 'listened', 'count(*) as col_count' ],
                      where => { artist => 'Jamiroquai', mime_type => 'audio/mpeg' },
                      group_by => 'listened',
                      order_by => "col_count desc",
                      having  => { col_count => {'<' => 5 }},

=cut

sub select {
  my $self = shift;
  if (! ref $_[1]) {
    # old style:
    return $self->SUPER::select(@_);
  } else {
    my ($table,$columns,%clauses) = @_;
    my $where = delete $clauses{where};
    if (exists $clauses{super_search}) {
      #warn Dumper \%clauses;
      my $value = delete $clauses{super_search};
      my $columns = delete $clauses{super_search_columns};
      my $op = delete $clauses{super_search_operator};
      $where = $self->super_search($value,$columns,$where,$op);
    };
    $clauses{where} = $where;

    return $self->SUPER::select($table,$columns,%clauses);
  };
};

1;

=head1 AUTHOR

Max Maischein, E<lt>corion@cpan.orgE<gt>

=head1 SEE ALSO

L<SQL::Abstract>, L<SQL::Abstract::Clauses>, L<SQL::Abstract::Limit>
