package Table::ColumnRefinement;
use strict;
use base 'Class::Accessor';
use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

Table::ColumnRefinement - Proposed subdivision of the values of a column

=head1 ABSTRACT

This class encapsulates the distribution of values in a column,
together with some helper methods.

(Name is subject to aprupt change)

=cut

__PACKAGE__->mk_accessors(qw(column values res_count));

use Data::Dumper;
use overload '""' => sub { shift->column }, fallback => 1;

sub new {
  my ($class,$table,$column,$count,$query);
  if (@_ == 4) {
    my $res;
    ($class,$table,$column,$res) = @_;
    $count = @$res;
    $query = $res->query;
  } else {
    ($class,$table,$column,$count,$query) = @_;
  };
  my $s = $table->sql;
  my $dbh = $table->dbh;

  my ($sql,@bind) = $s->select($table->table,[ $column, 'count(*) as col_count' ],
                          where => $query,
                          group_by => $column,
                          order_by => "col_count desc",
                          having  => { 'count(*)' => {'<' => $count }},
                          # having  => { 'count(*)' => {'>' => 0 }},
                          );

  my $sth = $dbh->prepare_cached( $sql );
  $sth->execute(@bind);
  my $res = $sth->fetchall_arrayref();
  my $values = [ map { bless +{ value => $_->[0], count => $_->[1] }, 'Table::ColumnRefinement::Value' } @$res ];

  my $self = {
    column => $column,
    values => $values,
    res_count => $count,
  };
  bless $self,$class;

  $self;
};

=head2 C<< is_implicit >>

Returns true if all rows in refinement have the
same value for this column.

=cut

sub is_implicit { @{shift->{values}} == 1 };

=head2 C<< is_trivial >>

Returns true if the refinement actually is no refinement
because it either all rows have the same column value
or all rows have a different column value.

In fact, this method returns one of two strings, depending
on I<how> triviality is reached:

C<implicit> means that all rows in the refinement have the
same value for the column.

C<trivial> means that all rows have a different value
from each other for the column.

In fact, C<is_trivial> should look at the query too, to
determine of a value is C<implicit> or C<explicit>,
that is, given in the query already. But that goes too far
for the moment.

=cut

sub is_trivial {
  my ($self) = @_;

  $self->is_implicit && return "implicit";

  # the counts are ordered by size
  ($self->{values}->[0]->{count} || 0 )<= 1 && return "trivial";

  return;
};

=head2 C<< is_partition >>

Returns true if the different values of the column
partition the whole result set, that is, there
are no NULL values in the column for the resultset.

=cut

sub is_partition { die "'is_partition' is unimplemented" };

# This should be somewhere else, like in a Template::Filter
sub css_class {
  my $result = $_[0]->column;
  $result =~ s![^\w\s]!!g;
  $result =~ s!\s+!_!g;
  $result
};

sub value_list { warn "Returning values"; @{ shift->values } };

=head1 TODO

There should be comparison operators implementing
subset semantics, that is, the should check whether
two column refinements are equal (that is, they
result in exactly the same subdivisions), or one
is finer than the other (that is, each subset of one
refinement is completely contained within a subset
of the other, but the refinements are not equal),
or undefined (none of the above).

This would create a half-order on the refinements.

=cut

package Table::ColumnRefinement::Value;
use strict;
use overload
  '""' => sub { shift->{value} },
  fallback => 1;

1;
