package Table::Denormalized::Manager;
use strict;
use vars qw($VERSION);
$VERSION = '0.06';

=head1 NAME

Table::Denormalized::Manager - base class for creating and maintaining dynamic tables

=head1 SYNOPSIS

This package provides a convenience constructor
for creating or altering a denormalized table.
The arguments are discussed below.

  package My::Index;
  use base 'Table::Denormalized::Manager';

  # ... time passes

  package main;

  my $idx = My::Index->new({
      dsn => 'dbi:SQLite:dbname=:memory:',
      table => 'my_table',
      verbose => 1,
      create_if_needed => 1,
  });

=head1 ACCESSORS

=head2 C<< table >>

Gets and sets the L<Table::Denormalized> instance.

=head2 C<< verbose >>

Gets and sets the output
of progress information. A true value
will make the manager output information.

=cut

use UNIVERSAL::require; # ugh, I know ...
use Scalar::Util qw( weaken );
use base 'Class::Accessor';
__PACKAGE__->mk_accessors(qw(table verbose));

=head1 METHODS

=head2 C<< new ARGS >>

Creates a new manager instance. The following
arguments are recognized:

=head3 C<create>

Creates the table. Wipes all data from the table
should it already exist.

=head3 C<create_if_needed>

Creates the table if needed.

=head3 C<table>

The name of the table where the index gets stored in. Default
is C<fileindex>.

=head3 C<dsn>

The DBI DSN to use. Default is C<dbi:SQLite:dbname=$0.sqlite>.

If you want an alternate database to be used,
you can change that here, or with the C<dbh> parameter,
if you already have a database handle.

=head3 C<dbh>

If you want to provide an alternate database
other than SQLite, you can supply the C<$dbh> you get
from a DBI connect call.

=head3 C<row_class>

The class into which returned rows are blessed into.
This allows you to specify a L<Class::DBI> or L<DBIx::Class>
subclass. The default is L<Table::Denormalized::Row>.

=head3 C<verbose>

Outputs actions to the screen.

=head3 C<managed_class>

The class to instantiate. The default
is C<Table::Denormalized> but this allows
you to specify a different class. The
package will be loaded using L<UNIVERSAL::require>.

=head3 C<columnset>

The column set(s) that the table should contain. If
this is a reference to an unblessed array it's
interpreted as a list of column sets, otherwise
it's interpreted as a single column set.

=cut

sub new {
    my ($class,$args) = @_;

    $args->{ table } ||= 'fileindex';
    $args->{ create_if_needed } ||= 1;
    $args->{ verbose } ||= 0;
    $args->{ columnset } ||= [];
    $args->{ managed_class } ||= 'Table::Denormalized';
    
    my $verbose = $args->{ verbose };

    $args->{managed_class}->require()
        or die "Couldn't load '$args->{managed_class}': $@";

    if (! ref $args->{table}) {
        # Only a table name was given
        my $dbh = delete $args->{dbh};
        if (! $dbh) {
            my $dsn = delete $args->{dsn} || ("dbi:SQLite:dbname=$0.sqlite");
            if ($verbose) {
                warn "Connecting to $dsn";
            };
            $dbh = DBI->connect($dsn, delete $args->{user}, delete $args->{password});
        };

        $args->{table} = $args->{managed_class}->new(
            dbh       => $dbh,
            table     => delete $args->{table},
            row_class => delete $args->{row_class},
        );
    };

    my $create = delete $args->{create};
    my $create_if_needed = delete $args->{create_if_needed};
    my @columns = ref $args->{columnset} eq 'ARRAY'
                  ? @{ $args->{columnset} }
                  : $args->{columnset};

    my $self = $class->SUPER::new($args);

    $self->table->add_set(@columns);

    $create ||= ! $self->table_exists
        if $create_if_needed;

    if ($create) {
        warn "Creating table $args->{table}"
          if $verbose;
        $self->table->create;
    };

    warn "Altering table $args->{table}"
        if $verbose;
    $self->table->alter_table($verbose);

    if ($verbose) {
        warn "Available columns are:" . join ",", map {$_->name} ($self->table->columns);
    };

    $self
}

=head2 C<< query PARAMS >>

Pass through to L<Table::Denormalized>

=cut

sub query {
    my ($self,$query) = @_;
    $self->query_ex(where => $query);
};

=head2 C<< all >>

Shorthand for C<< $query->query({}) >>.
It returns all items in the table.

=cut

sub all { shift->query({}); };

=head2 C<< query_ex PARAMS >>

Pass through to L<Table::Denormalized>

=cut

sub query_ex {
    my ($self,%args) = @_;
    my $results = $self->table->query_ex(%args);
    
    my $weak = $self;
    weaken $weak;
    
    $_->{_index} = $weak
        for (@$results);
    
    $results
};

=head2 C<< $mgr->table_exists [NAME] >>

Returns true if the table C<NAME> exists
in the database.

If C<NAME> is not given, the default table
name is assumed.

=cut

sub table_exists {
  my ($self,$name) = @_;
  if (! defined $name) {
      $name = $self->table->table;
  }
  
  my $dbh = $self->table->dbh;
  local $dbh->{PrintError};
  local $dbh->{RaiseError} = 1;
  eval {
    my $sth = $dbh->prepare(sprintf 'select * from %s where 1 = 0', $name);
    $sth->execute()
  };
  $@ ? 0 : 1
};

=head2 C<< columns >>

Returns the columns of the resultset.

=cut

sub columns { shift->table->columns };

=head2 C<< sql >>

Returns the L<SQL::Abstract::Clauses> object of the table.

=cut

sub sql { shift->table->sql };

=head2 C<< dbh >>

Returns the database handle of the table.

=cut

sub dbh { shift->table->dbh };

=head2 C<< retrieve ITEMS >>

Retrieves rows as given by their primary keys.

=cut

# This smells like FFRIndexed shouldn't delegate
# to but inherit from Table::Denormalized ...
sub retrieve {
  my ($self) = shift;
  $self->query( { '_id' => { -in => \@_ }} )
};

1;

=head1 AUTHOR

Max Maischein, E<lt>corion@cpan.orgE<gt>

=head1 SEE ALSO

L<Table::Denormalized>
